@@ -40,6 +40,7 @@ import com.verisign.epp.serverstub.v06.FeeV06DomainHandler;
 import com.verisign.epp.serverstub.v07.FeeV07DomainHandler;
 import com.verisign.epp.serverstub.v08.FeeV08DomainHandler;
 import com.verisign.epp.serverstub.v09.FeeV09DomainHandler;
+import com.verisign.epp.serverstub.v11.FeeV11DomainHandler;
 import com.verisign.epp.util.EPPCatFactory;
 
 /**
@@ -56,8 +57,9 @@ import com.verisign.epp.util.EPPCatFactory;
 public class FeeDomainHandler extends DomainHandler {
 
 	/** Used for logging */
-	private static Logger cat = Logger.getLogger(FeeDomainHandler.class
-			.getName(), EPPCatFactory.getInstance().getFactory());
+	private static Logger cat = Logger.getLogger(
+			FeeDomainHandler.class.getName(),
+			EPPCatFactory.getInstance().getFactory());
 
 	// Sub-handler for handling version 0.6 extension messages.
 	FeeV06DomainHandler v06SubHandler = new FeeV06DomainHandler();
@@ -71,6 +73,9 @@ public class FeeDomainHandler extends DomainHandler {
 	// Sub-handler for handling version 0.9 extension messages.
 	FeeV09DomainHandler v09SubHandler = new FeeV09DomainHandler();
 
+	// Sub-handler for handling version 0.9 extension messages.
+	FeeV11DomainHandler v11SubHandler = new FeeV11DomainHandler();
+
 	/**
 	 * Invoked when a Domain Check command is received.
 	 *
@@ -89,25 +94,30 @@ public class FeeDomainHandler extends DomainHandler {
 		EPPEventResponse theEventResponse = super.doDomainCheck(aEvent, aData);
 
 		// v06 of fee check?
-		if (theCommand
-				.hasExtension(com.verisign.epp.codec.fee.v06.EPPFeeCheck.class)) {
+		if (theCommand.hasExtension(
+				com.verisign.epp.codec.fee.v06.EPPFeeCheck.class)) {
 			this.v06SubHandler.doDomainCheck(theCommand,
 					(EPPResponse) theEventResponse.getResponse(), aData);
 		} // v07 of fee check?
-		else if (theCommand
-				.hasExtension(com.verisign.epp.codec.fee.v07.EPPFeeCheck.class)) {
+		else if (theCommand.hasExtension(
+				com.verisign.epp.codec.fee.v07.EPPFeeCheck.class)) {
 			this.v07SubHandler.doDomainCheck(theCommand,
 					(EPPResponse) theEventResponse.getResponse(), aData);
 		} // v08 of fee check?
-		else if (theCommand
-				.hasExtension(com.verisign.epp.codec.fee.v08.EPPFeeCheck.class)) {
+		else if (theCommand.hasExtension(
+				com.verisign.epp.codec.fee.v08.EPPFeeCheck.class)) {
 			this.v08SubHandler.doDomainCheck(theCommand,
 					(EPPResponse) theEventResponse.getResponse(), aData);
 		} // v09 of fee check?
-		else if (theCommand
-				.hasExtension(com.verisign.epp.codec.fee.v09.EPPFeeCheck.class)) {
+		else if (theCommand.hasExtension(
+				com.verisign.epp.codec.fee.v09.EPPFeeCheck.class)) {
 			this.v09SubHandler.doDomainCheck(theCommand,
 					(EPPResponse) theEventResponse.getResponse(), aData);
+		} // v11 of fee check?
+		else if (theCommand.hasExtension(
+				com.verisign.epp.codec.fee.v11.EPPFeeCheck.class)) {
+			this.v11SubHandler.doDomainCheck(theCommand,
+					(EPPResponse) theEventResponse.getResponse(), aData);
 		}
 		else {
 			cat.info("doDomainCheck: fee check = null");
@@ -135,16 +145,16 @@ public class FeeDomainHandler extends DomainHandler {
 		EPPEventResponse theEventResponse = super.doDomainInfo(aEvent, aData);
 
 		// v06 of fee info?
-		if (theCommand
-				.hasExtension(com.verisign.epp.codec.fee.v06.EPPFeeInfo.class)) {
+		if (theCommand.hasExtension(
+				com.verisign.epp.codec.fee.v06.EPPFeeInfo.class)) {
 			this.v06SubHandler.doDomainInfo(theCommand,
 					(EPPResponse) theEventResponse.getResponse(), aData);
 		} // v07 of fee info?
-		else if (theCommand
-				.hasExtension(com.verisign.epp.codec.fee.v07.EPPFeeInfo.class)) {
+		else if (theCommand.hasExtension(
+				com.verisign.epp.codec.fee.v07.EPPFeeInfo.class)) {
 			this.v07SubHandler.doDomainInfo(theCommand,
 					(EPPResponse) theEventResponse.getResponse(), aData);
-		} 
+		}
 		else {
 			cat.info("doDomainInfo: fee info = null");
 			System.out.println("doDomainInfo: fee info = null");
@@ -172,25 +182,30 @@ public class FeeDomainHandler extends DomainHandler {
 		EPPEventResponse theEventResponse = super.doDomainCreate(aEvent, aData);
 
 		// v06 of fee create?
-		if (theCommand
-				.hasExtension(com.verisign.epp.codec.fee.v06.EPPFeeCreate.class)) {
+		if (theCommand.hasExtension(
+				com.verisign.epp.codec.fee.v06.EPPFeeCreate.class)) {
 			this.v06SubHandler.doDomainCreate(theCommand,
 					(EPPResponse) theEventResponse.getResponse(), aData);
 		} // v07 of fee create?
-		else if (theCommand
-				.hasExtension(com.verisign.epp.codec.fee.v07.EPPFeeCreate.class)) {
+		else if (theCommand.hasExtension(
+				com.verisign.epp.codec.fee.v07.EPPFeeCreate.class)) {
 			this.v07SubHandler.doDomainCreate(theCommand,
 					(EPPResponse) theEventResponse.getResponse(), aData);
 		} // v08 of fee create?
-		else if (theCommand
-				.hasExtension(com.verisign.epp.codec.fee.v08.EPPFeeCreate.class)) {
+		else if (theCommand.hasExtension(
+				com.verisign.epp.codec.fee.v08.EPPFeeCreate.class)) {
 			this.v08SubHandler.doDomainCreate(theCommand,
 					(EPPResponse) theEventResponse.getResponse(), aData);
 		} // v09 of fee create?
-		else if (theCommand
-				.hasExtension(com.verisign.epp.codec.fee.v09.EPPFeeCreate.class)) {
+		else if (theCommand.hasExtension(
+				com.verisign.epp.codec.fee.v09.EPPFeeCreate.class)) {
 			this.v09SubHandler.doDomainCreate(theCommand,
 					(EPPResponse) theEventResponse.getResponse(), aData);
+		} // v11 of fee create?
+		else if (theCommand.hasExtension(
+				com.verisign.epp.codec.fee.v11.EPPFeeCreate.class)) {
+			this.v11SubHandler.doDomainCreate(theCommand,
+					(EPPResponse) theEventResponse.getResponse(), aData);
 		}
 		else {
 			cat.info("doDomainCreate: fee create = null");
@@ -224,6 +239,7 @@ public class FeeDomainHandler extends DomainHandler {
 		boolean hasV07ExtService = false;
 		boolean hasV08ExtService = false;
 		boolean hasV09ExtService = false;
+		boolean hasV11ExtService = false;
 		Enumeration extSvcEnum = theSessionData.getLoginCmd()
 				.getExtensionServices().elements();
 		while (extSvcEnum.hasMoreElements()) {
@@ -245,15 +261,19 @@ public class FeeDomainHandler extends DomainHandler {
 					com.verisign.epp.codec.fee.v09.EPPFeeExtFactory.NS)) {
 				hasV09ExtService = true;
 			}
+			if (theExtService.getNamespaceURI().equals(
+					com.verisign.epp.codec.fee.v11.EPPFeeExtFactory.NS)) {
+				hasV11ExtService = true;
+			}
 		}
 
 		/*
-		 *  Pick all handlers that apply.  This is not compliant with 
-		 *  section 2 "Migrating to Newer Versions of This Extension", 
-		 *  but is needed to support isolated client tests that expect 
-		 *  a specific extension version in the response.
-		 */		
-		
+		 * Pick all handlers that apply. This is not compliant with section 2
+		 * "Migrating to Newer Versions of This Extension", but is needed to
+		 * support isolated client tests that expect a specific extension
+		 * version in the response.
+		 */
+
 		// v06 of fee delete?
 		if (hasV06ExtService) {
 			this.v06SubHandler.doDomainDelete(theCommand,
@@ -278,10 +298,17 @@ public class FeeDomainHandler extends DomainHandler {
 					(EPPResponse) theEventResponse.getResponse(), aData);
 		}
 
-		if (!hasV06ExtService && !hasV07ExtService &!hasV08ExtService && !hasV09ExtService) {
+		// v11 of fee delete?
+		if (hasV11ExtService) {
+			this.v11SubHandler.doDomainDelete(theCommand,
+					(EPPResponse) theEventResponse.getResponse(), aData);
+		}
+
+		if (!hasV06ExtService && !hasV07ExtService & !hasV08ExtService
+				&& !hasV09ExtService && !hasV11ExtService) {
 			cat.info("doDomainDelete: fee extension not supported by client");
-			System.out
-					.println("doDomainDelete: fee extension not supported by client");
+			System.out.println(
+					"doDomainDelete: fee extension not supported by client");
 		}
 
 		return theEventResponse;
@@ -305,25 +332,30 @@ public class FeeDomainHandler extends DomainHandler {
 		EPPEventResponse theEventResponse = super.doDomainRenew(aEvent, aData);
 
 		// v06 of fee renew?
-		if (theCommand
-				.hasExtension(com.verisign.epp.codec.fee.v06.EPPFeeRenew.class)) {
+		if (theCommand.hasExtension(
+				com.verisign.epp.codec.fee.v06.EPPFeeRenew.class)) {
 			this.v06SubHandler.doDomainRenew(theCommand,
 					(EPPResponse) theEventResponse.getResponse(), aData);
 		} // v07 of fee renew?
-		else if (theCommand
-				.hasExtension(com.verisign.epp.codec.fee.v07.EPPFeeRenew.class)) {
+		else if (theCommand.hasExtension(
+				com.verisign.epp.codec.fee.v07.EPPFeeRenew.class)) {
 			this.v07SubHandler.doDomainRenew(theCommand,
 					(EPPResponse) theEventResponse.getResponse(), aData);
 		} // v08 of fee renew?
-		else if (theCommand
-				.hasExtension(com.verisign.epp.codec.fee.v08.EPPFeeRenew.class)) {
+		else if (theCommand.hasExtension(
+				com.verisign.epp.codec.fee.v08.EPPFeeRenew.class)) {
 			this.v08SubHandler.doDomainRenew(theCommand,
 					(EPPResponse) theEventResponse.getResponse(), aData);
 		} // v09 of fee renew?
-		else if (theCommand
-				.hasExtension(com.verisign.epp.codec.fee.v09.EPPFeeRenew.class)) {
+		else if (theCommand.hasExtension(
+				com.verisign.epp.codec.fee.v09.EPPFeeRenew.class)) {
 			this.v09SubHandler.doDomainRenew(theCommand,
 					(EPPResponse) theEventResponse.getResponse(), aData);
+		} // v11 of fee renew?
+		else if (theCommand.hasExtension(
+				com.verisign.epp.codec.fee.v11.EPPFeeRenew.class)) {
+			this.v11SubHandler.doDomainRenew(theCommand,
+					(EPPResponse) theEventResponse.getResponse(), aData);
 		}
 		else {
 			cat.info("doDomainRenew: fee renew = null");
@@ -358,6 +390,7 @@ public class FeeDomainHandler extends DomainHandler {
 		boolean hasV07ExtService = false;
 		boolean hasV08ExtService = false;
 		boolean hasV09ExtService = false;
+		boolean hasV11ExtService = false;
 		Enumeration extSvcEnum = theSessionData.getLoginCmd()
 				.getExtensionServices().elements();
 		while (extSvcEnum.hasMoreElements()) {
@@ -382,18 +415,22 @@ public class FeeDomainHandler extends DomainHandler {
 					com.verisign.epp.codec.fee.v09.EPPFeeExtFactory.NS)) {
 				hasV09ExtService = true;
 			}
+			
+			if (theExtService.getNamespaceURI().equals(
+					com.verisign.epp.codec.fee.v11.EPPFeeExtFactory.NS)) {
+				hasV11ExtService = true;
+			}
 		}
 
 		// Transfer Query?
 		if (theCommand.getOp().equals(EPPCommand.OP_QUERY)) {
 
 			/*
-			 *  Pick all handlers that apply.  This is not compliant with 
-			 *  section 2 "Migrating to Newer Versions of This Extension", 
-			 *  but is needed to support isolated client tests that expect 
-			 *  a specific extension version in the response.
+			 * Pick all handlers that apply. This is not compliant with section
+			 * 2 "Migrating to Newer Versions of This Extension", but is needed
+			 * to support isolated client tests that expect a specific extension
+			 * version in the response.
 			 */
-			
 
 			// v06 of transfer query?
 			if (hasV06ExtService) {
@@ -419,40 +456,53 @@ public class FeeDomainHandler extends DomainHandler {
 						(EPPResponse) theEventResponse.getResponse(), aData);
 			}
 
-			if (!hasV06ExtService && !hasV07ExtService && !hasV08ExtService && !hasV09ExtService) {
-				cat.info("doDomainTransfer: fee extension not supported by client");
-				System.out
-						.println("doDomainTransfer: fee extension not supported by client");
+			// v11 of transfer query?
+			if (hasV11ExtService) {
+				this.v11SubHandler.doDomainTransfer(theCommand,
+						(EPPResponse) theEventResponse.getResponse(), aData);
+			}
+
+			if (!hasV06ExtService && !hasV07ExtService && !hasV08ExtService
+					&& !hasV09ExtService && !hasV11ExtService) {
+				cat.info(
+						"doDomainTransfer: fee extension not supported by client");
+				System.out.println(
+						"doDomainTransfer: fee extension not supported by client");
 			}
 
 		}
 		else if (theCommand.getOp().equals(EPPCommand.OP_REQUEST)) {
 
 			// v06 of fee transfer request?
-			if (theCommand
-					.hasExtension(com.verisign.epp.codec.fee.v06.EPPFeeTransfer.class)) {
+			if (theCommand.hasExtension(
+					com.verisign.epp.codec.fee.v06.EPPFeeTransfer.class)) {
 				this.v06SubHandler.doDomainTransfer(theCommand,
 						(EPPResponse) theEventResponse.getResponse(), aData);
 			} // v07 of fee transfer request?
-			else if (theCommand
-					.hasExtension(com.verisign.epp.codec.fee.v07.EPPFeeTransfer.class)) {
+			else if (theCommand.hasExtension(
+					com.verisign.epp.codec.fee.v07.EPPFeeTransfer.class)) {
 				this.v07SubHandler.doDomainTransfer(theCommand,
 						(EPPResponse) theEventResponse.getResponse(), aData);
 			} // v08 of fee transfer request?
-			else if (theCommand
-					.hasExtension(com.verisign.epp.codec.fee.v08.EPPFeeTransfer.class)) {
+			else if (theCommand.hasExtension(
+					com.verisign.epp.codec.fee.v08.EPPFeeTransfer.class)) {
 				this.v08SubHandler.doDomainTransfer(theCommand,
 						(EPPResponse) theEventResponse.getResponse(), aData);
 			} // v09 of fee transfer request?
-			else if (theCommand
-					.hasExtension(com.verisign.epp.codec.fee.v09.EPPFeeTransfer.class)) {
+			else if (theCommand.hasExtension(
+					com.verisign.epp.codec.fee.v09.EPPFeeTransfer.class)) {
 				this.v09SubHandler.doDomainTransfer(theCommand,
 						(EPPResponse) theEventResponse.getResponse(), aData);
-			} 
+			} // v11 of fee transfer request?
+			else if (theCommand.hasExtension(
+					com.verisign.epp.codec.fee.v11.EPPFeeTransfer.class)) {
+				this.v11SubHandler.doDomainTransfer(theCommand,
+						(EPPResponse) theEventResponse.getResponse(), aData);
+			}
 			else {
 				cat.info("doDomainTransfer: fee transfer request = null");
-				System.out
-						.println("doDomainTransfer: fee transfer request = null");
+				System.out.println(
+						"doDomainTransfer: fee transfer request = null");
 			}
 		}
 
@@ -478,25 +528,30 @@ public class FeeDomainHandler extends DomainHandler {
 		EPPEventResponse theEventResponse = super.doDomainUpdate(aEvent, aData);
 
 		// v06 of fee update?
-		if (theCommand
-				.hasExtension(com.verisign.epp.codec.fee.v06.EPPFeeUpdate.class)) {
+		if (theCommand.hasExtension(
+				com.verisign.epp.codec.fee.v06.EPPFeeUpdate.class)) {
 			this.v06SubHandler.doDomainUpdate(theCommand,
 					(EPPResponse) theEventResponse.getResponse(), aData);
 		} // v07 of fee update?
-		else if (theCommand
-				.hasExtension(com.verisign.epp.codec.fee.v07.EPPFeeUpdate.class)) {
+		else if (theCommand.hasExtension(
+				com.verisign.epp.codec.fee.v07.EPPFeeUpdate.class)) {
 			this.v07SubHandler.doDomainUpdate(theCommand,
 					(EPPResponse) theEventResponse.getResponse(), aData);
 		} // v08 of fee update?
-		else if (theCommand
-				.hasExtension(com.verisign.epp.codec.fee.v08.EPPFeeUpdate.class)) {
+		else if (theCommand.hasExtension(
+				com.verisign.epp.codec.fee.v08.EPPFeeUpdate.class)) {
 			this.v08SubHandler.doDomainUpdate(theCommand,
 					(EPPResponse) theEventResponse.getResponse(), aData);
 		} // v09 of fee update?
-		else if (theCommand
-				.hasExtension(com.verisign.epp.codec.fee.v09.EPPFeeUpdate.class)) {
+		else if (theCommand.hasExtension(
+				com.verisign.epp.codec.fee.v09.EPPFeeUpdate.class)) {
 			this.v09SubHandler.doDomainUpdate(theCommand,
 					(EPPResponse) theEventResponse.getResponse(), aData);
+		} // v11 of fee update?
+		else if (theCommand.hasExtension(
+				com.verisign.epp.codec.fee.v11.EPPFeeUpdate.class)) {
+			this.v11SubHandler.doDomainUpdate(theCommand,
+					(EPPResponse) theEventResponse.getResponse(), aData);
 		}
 		else {
 			cat.info("doDomainUpdate: fee update = null");
